%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Algoriothm 2 (strictly causal version)
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : 
%         tap_set	: [ source filter tap ; relay filter tap ]
%         s     : Noise standard deviation
%         N 	: Number of partition to calculate the integration
%         epsilon : The maximum relative error between the previous point and the current point. If the error exceeds epsilon, then the function terminates. 
%         max_iter : The maximum number of iteration to avoid the infinite loop.
%         lambda : Step size coefficient for the subgradient
%         L : Delay spread
%         h_sr : FIR channel between the source and the relay; (L) by (1)
%         h_sd : FIR channel between the source and the destination; (L) by (1)
%         h_rd : FIR channel between the relay and the destination; (L) by (1)
%         Ps : Source power
%         Pr : Relay power
%         u_initial : Initial filter coefficients of size (LsxLr) by (1)

% Output :      
%         rate_normal    :  The set of rate at each iteration ; (max_iter)by(1)
%         u_opt : Optimal filter coefficients of size (LsxLr) by (1)

%%% For strictly causal filtering, the subgradient and the projection are
%%% different.



function [rate_normal , u_opt] = func_algorithm2_strict( parameters )

%parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial ] ;
tap_set=parameters(1:2) ;
s = parameters(3) ;
N = parameters(4) ;
epsilon = parameters(5) ;
max_iter = parameters(6) ;
lambda = parameters(7) ;
L =  parameters(8) ;
h_sr = parameters(9:9+L-1) ;
h_sd= parameters(9+L : 9+L+L-1) ;
h_rd= parameters(9+2*L : 9+2*L+L-1) ;
Ps= parameters(9+3*L) ;
Pr= parameters(9+3*L+1) ;
u_initial = parameters(11+3*L: 11+3*L+sum(tap_set)-1) ;

rate_normal = zeros( max_iter , 1) ; % result table

Ls = tap_set(1) ;  % source filter taps
Lr = tap_set(2) ; % relay filter taps

u_initial_on_C1 = func_project_onto_C1(Ps, Ls ,  u_initial) ; % projection to C1
u_initial_on_C1(1) = 0 ;
u_initial_on_C1(Ls+1) = 0 ;
[u_unitial_on_C1_and_C2, flag] = function_project_onto_C2_strictly(real(u_initial_on_C1), Pr, s, Ls, Lr, N, h_sr) ; % projection to C2                                                        
u_initial = real(u_unitial_on_C1_and_C2) ;  

u = real(u_initial) ; % initial condition

%%% program

iter = 0 ;
err = zeros(max_iter, 1 ) ;
% residual_grad = zeros(max_iter ,1) ;

err_flag = 1 ;
fprintf(1,'\n\nIter\t\tError flag\tRate\n') ;

while  (iter < max_iter)  && ( err_flag > epsilon ) 

    iter = iter + 1 ;

    u_current = u ;
    u(1) = 0 ; % strictly causal
    u(Ls+1) = 0 ; % strictly causal
    [rate, grad_vec] = func_subgrad(h_sd, h_rd, h_sr , N, s, Ls, Lr, u) ;
    grad_vec = real(grad_vec) ;

    % diminishing step size.
    step = lambda/sqrt(iter) * (-rate)/ (transpose(-grad_vec)*(-grad_vec)) ; % normalized instant steepesat descent (minus comes from minimization framework)
    u_next = u_current - real(step)*grad_vec ; % unconstrained update

    %  C1-> C2
    u_on_C1 = func_project_onto_C1(Ps, Ls ,u_next ) ; % projection to C1
    u_on_C1 = real(u_on_C1) ;
    
    [ u_on_C1_and_C2, flag] = function_project_onto_C2_strictly(u_on_C1, Pr, s, Ls, Lr, N, h_sr) ; % projection to C2
    u_on_C1_and_C2 = real(u_on_C1_and_C2) ;
    u_on_C1_and_C2(1) = 0 ; % strictly causal
    u_on_C1_and_C2(Ls+1) = 0 ; % strictly causal

    [rate_after_projection, grad_vec_after_projection] = func_subgrad(h_sd, h_rd, h_sr , N, s, Ls, Lr, u_on_C1_and_C2) ; % call for rate calculation
 
    err(iter) = transpose(u_on_C1_and_C2 - u_current)*(u_on_C1_and_C2 - u_current) / (transpose(u_current)*u_current);  % if it is the fixed point, then stop    

    err_flag = err(iter) ;
    u = u_on_C1_and_C2 ;  % final solution is called u
    [u_opt] = u ;
    fprintf(1,'%5d%20e%10f\n',iter , err_flag, real(rate_after_projection)) ;
    rate_normal(iter) =real(rate_after_projection) ;  % we will run in one set only when we control power.

end

fprintf(1,'\n\nError flag=%20e\nRate=%10f\nPs=%5f,\tPr=%5f\n\n',err_flag , real(rate_after_projection), Ps, Pr ) ;

