%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Subgradient and the rate
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : 
%         N 	: Number of partition to calculate the integration
%         u : The point for subgradient; (Lr+Ls) by (1)
%         h_sr : FIR channel between the source and the relay; (L) by (1)
%         h_sd : FIR channel between the source and the destination; (L) by (1)
%         h_rd : FIR channel between the relay and the destination; (L) by (1)
%         s     : Noise standard deviation
%         Ls  : Number of source filter taps
%         Lr  : Number of relay filter taps
% Output :      
%         rate    :  Achievable rate at the point u
%         grad_vec    :  Gradient vector at the point u ;  (Lr+Ls) by (1)

function [rate , grad_vec ] = func_subgrad(h_sd, h_rd, h_sr , N, s, Ls, Lr, u)

t = u(1:Ls) ; % source filter
h =u(1+Ls:end) ; % relay filter

% Function 'lgwt' is due to Greg von Winckel - 02/25/2004
[w_sampled,weight] = lgwt(N, 0, 2*pi ) ; % freq bins to use quadrature

grad_val = zeros(Ls+Lr,N) ; % store gradient value
rate_val = zeros(N,1) ; % store rate value

for id_carr = 1 : N
   
    w = w_sampled(id_carr) ; % specific freq bin
    w_t = exp(1i*w*transpose([0:Ls-1])) ;
    w_h = exp(1i*w*transpose([0:Lr-1])) ;    
    
    dft_vec = exp(1i*w*transpose([0:N-1])) ; % dft vector
    
    H_sd = ctranspose(dft_vec) * [h_sd ; zeros(N-length(h_sd),1) ] ; % freq value at this bin
    H_sr = ctranspose(dft_vec) * [h_sr ; zeros(N-length(h_sr),1) ] ; % freq value at this bin
    H_rd = ctranspose(dft_vec) * [h_rd ; zeros(N-length(h_rd),1) ] ; % freq value at this bin
    
    H =  ctranspose(w_h) * h ; % freq value at this bin
        
    Wt = w_t*ctranspose(w_t) + conj(w_t)*transpose(w_t) ;
    Wh = w_h*ctranspose(w_h) + conj(w_h)*transpose(w_h) ;
    
    num = abs(H_sd + H_sr*H*H_rd)^2 ;
    denom = (abs(H_rd*H)^2+1)*s^2 ;
    CNR = num/denom ; % CNR

    D_num = abs(H_sr)^2*abs(H_rd)^2*Wh*h + H_sd * conj(H_sr) * conj(H_rd)*w_h + conj(H_sd)*H_sr*H_rd*conj(w_h) ; % num'
    D_denom = abs(H_rd)^2*Wh*h*s^2 ;    % denom'
    
    X = CNR* transpose(t) * w_t * ctranspose(w_t) * t ;            
    Y = CNR * Wt*t ;  
    Z1 = transpose(t) * w_t * ctranspose(w_t) * t / denom ;
    Z2 = D_num ;
    Z3 = D_denom ;
    Z4 = CNR ;    

    grad_val(:,id_carr) = 1/2/log(2) / (1 + X) * [ Y ; Z1*(Z2-Z3*Z4)] * weight(id_carr) ; % grad of rate ; which is weighted by quadrature coefficient
    rate_val(id_carr) = 1/2*log2(1 + CNR * transpose(t)*w_t*ctranspose(w_t)*t ) ; % rate itself    
    
end

grad_vec = sum(grad_val,2)/2/pi ;
rate = sum(rate_val.*weight)/2/pi ;

