%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Obtain the ellipse to be projected
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : 
%         u : The point for subgradient; (Lr+Ls) by (1)
%         Pr : The relay power
%         s     : Noise standard deviation
%         Ls  : Number of source filter taps
%         Lr  : Number of relay filter taps
%         N 	: Number of partition to calculate the integration
%         h_sr : FIR channel between the source and the relay; (L) by (1)
% Output :      
%         uu    :  The point after projection onto the 2nd constraint set ; (Lr+Ls) by (1) 
%         flag    :  error flag


function [uu , flag] = function_project_onto_C2_strictly(u, Pr, s, Ls, Lr, N, h_sr)

u(1) = 0 ; % strict causal
u(Ls+1) = 0 ; % strict causal

t = u(1:Ls) ; % source filter
h = u(Ls+1 : end) ; % relay filter

% Function 'lgwt' is due to Greg von Winckel - 02/25/2004
[w_sampled,weight] = lgwt(N, 0, 2*pi ) ; % freq bins to use quadrature
dft_mat_quad = exp(1i).^[ transpose([0:N-1]) * transpose(w_sampled) ] ; % dft matrix at sampling points on the unit circle

H_sr = ctranspose(dft_mat_quad) * [h_sr ; zeros(N-length(h_sr),1) ];  % freq representation of h_sr
T = ctranspose(dft_mat_quad) * [t ; zeros(N-length(t),1) ] ; % freq representation of t
H =  ctranspose(dft_mat_quad) * [h ; zeros(N-length(h),1) ] ; % freq representation of h

tmp1= abs(H_sr).^2 ;
tmp2 = abs(T).^2 ;
aa = tmp1 .* tmp2 + s^2 ; % alpha array

tmp1 = abs(H).^2 ;
tmp2 = abs(H_sr).^2 ;
tmp3 = abs(T).^2 ;
integrand = tmp1.*(tmp2.*tmp3+s^2) ; % integrand of constraint 2

if  sum(integrand.*weight)/2/pi < Pr  
    uu = u ; % if u is feasible, then stay still.
    flag_inner = 0 ;
    
else % if u is not feasible, then project.

    integrand_quad = zeros( Lr) ; % array quad for integrand2
    for id_bin = 1 : N
        w_val = w_sampled(id_bin) ;  % sampled point
        w_h=exp(1i*w_val*transpose([0:Lr-1]) ) ; % quasi-dft vector        
        integrand_quad = integrand_quad + w_h * ctranspose(w_h) * aa(id_bin) * weight(id_bin) ; % weight sum for quad integration
    end
    
    Q_quad = integrand_quad/2/pi ;  % ellipse equation
    Q_quad_causal = Q_quad(2:end, 2:end) ; % strict causal means that we must reduce the size of matrix
    [h0, flag_inner] = project_onto_ellipse(Q_quad_causal/Pr ,h(2:Lr)) ; % the actual projection

    uu = u ;
    uu(Ls+1 : end) = [ 0 ; h0 ] ; % update h0 ; causality
        
end

flag = flag_inner ; 
    
