%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Projection onto the ellipsoid
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : 
%         Q : the Hermitian positive semidefinite matrix
%         h : The point to be projected
% Output :      
%         h0    :  The point after projection onto the ellipsoid defined from Q
%         flag    :  error flag

function [h0, flag] = project_onto_ellipse(Q ,h)

[ A , Sig ] = eig(Q) ;
AA = pinv(A) ;
[Lr, dum] = size(Q) ;

epsilon = 1e-4 ;
iter = 0 ;

mu = 0 ; % initial point
mu_prev = 1 ;
err = (mu-mu_prev)/mu_prev ; 

while ( abs(err) > epsilon)   % newton method
    
    tmp_num = zeros(Lr,1) ;
    tmp_denom = zeros(Lr,1) ;

    for id_dim = 1 : Lr
        lambda = Sig(id_dim,id_dim) ; % eigenvalue
        tmp_num(id_dim) = lambda / ((1+mu *lambda)^2) ;
        tmp_denom(id_dim) = -2*lambda^2/ ((1+mu* lambda)^3) ;
    end
    
    step_num = transpose(h)*A*diag(tmp_num) * AA * h - 1 ;
    step_denom = transpose(h)*A* diag(tmp_denom) * AA * h ;
        
    step = step_num / step_denom ; % update size
    
    mu = mu - step ;   
    iter = iter + 1 ; % newton iteration
    
    err = (mu-mu_prev)/mu_prev ; % relative error
    mu_prev = mu ;
    
end

h0 = real( pinv(eye(Lr) + mu* Q) * h) ; % metric projection 
flag = ( transpose(h0) * Q * h0 > 1) ;





